/** MasterGear: portable MasterSystem/GameGear emulator ******/
/**                                                         **/
/**                        Common.h                         **/
/**                                                         **/
/** This file contains parts of the drivers which are       **/
/** common for all 256-color byte-per-pixel systems.        **/
/**                                                         **/
/** Copyright (C) Marat Fayzullin 1996                      **/
/**     You are not allowed to distribute this software     **/
/**     commercially. Please, notify me, if you make any    **/   
/**     changes to this file.                               **/
/*************************************************************/

static void RefreshSprites(byte Y,byte *Z);

/** RefreshLine() ********************************************/
/** Refresh line Y (0..191), including sprites in this line.**/
/*************************************************************/
void RefreshLine(register byte Y)
{
  /* Conversion matrix where result has bits flipped over */
  static byte Conv[256] =
  {
    0x00,0x80,0x40,0xC0,0x20,0xA0,0x60,0xE0,0x10,0x90,0x50,0xD0,
    0x30,0xB0,0x70,0xF0,0x08,0x88,0x48,0xC8,0x28,0xA8,0x68,0xE8,
    0x18,0x98,0x58,0xD8,0x38,0xB8,0x78,0xF8,0x04,0x84,0x44,0xC4,
    0x24,0xA4,0x64,0xE4,0x14,0x94,0x54,0xD4,0x34,0xB4,0x74,0xF4,
    0x0C,0x8C,0x4C,0xCC,0x2C,0xAC,0x6C,0xEC,0x1C,0x9C,0x5C,0xDC,
    0x3C,0xBC,0x7C,0xFC,0x02,0x82,0x42,0xC2,0x22,0xA2,0x62,0xE2,
    0x12,0x92,0x52,0xD2,0x32,0xB2,0x72,0xF2,0x0A,0x8A,0x4A,0xCA,
    0x2A,0xAA,0x6A,0xEA,0x1A,0x9A,0x5A,0xDA,0x3A,0xBA,0x7A,0xFA,
    0x06,0x86,0x46,0xC6,0x26,0xA6,0x66,0xE6,0x16,0x96,0x56,0xD6,
    0x36,0xB6,0x76,0xF6,0x0E,0x8E,0x4E,0xCE,0x2E,0xAE,0x6E,0xEE,
    0x1E,0x9E,0x5E,0xDE,0x3E,0xBE,0x7E,0xFE,0x01,0x81,0x41,0xC1,
    0x21,0xA1,0x61,0xE1,0x11,0x91,0x51,0xD1,0x31,0xB1,0x71,0xF1,
    0x09,0x89,0x49,0xC9,0x29,0xA9,0x69,0xE9,0x19,0x99,0x59,0xD9,
    0x39,0xB9,0x79,0xF9,0x05,0x85,0x45,0xC5,0x25,0xA5,0x65,0xE5,
    0x15,0x95,0x55,0xD5,0x35,0xB5,0x75,0xF5,0x0D,0x8D,0x4D,0xCD,
    0x2D,0xAD,0x6D,0xED,0x1D,0x9D,0x5D,0xDD,0x3D,0xBD,0x7D,0xFD,
    0x03,0x83,0x43,0xC3,0x23,0xA3,0x63,0xE3,0x13,0x93,0x53,0xD3,
    0x33,0xB3,0x73,0xF3,0x0B,0x8B,0x4B,0xCB,0x2B,0xAB,0x6B,0xEB,
    0x1B,0x9B,0x5B,0xDB,0x3B,0xBB,0x7B,0xFB,0x07,0x87,0x47,0xC7,
    0x27,0xA7,0x67,0xE7,0x17,0x97,0x57,0xD7,0x37,0xB7,0x77,0xF7,
    0x0F,0x8F,0x4F,0xCF,0x2F,0xAF,0x6F,0xEF,0x1F,0x9F,0x5F,0xDF,
    0x3F,0xBF,0x7F,0xFF
  };

  register byte I,M,J,K,*P,*T,*R,*C,*Z;
  register unsigned int L;
  byte X1,X2,XR,Offset,Shift;
  byte ZBuf[35];

  /* Calculate the starting address of a picture in XBuf, ZBuf */
  L=WIDTH*(HEIGHT-192)/2+(WIDTH-256)/2+WIDTH*Y;
  P=XBuf+L;Z=ZBuf+1;

  /* If display is turned off, fill scanline with black and exit */
  if(!ScreenON) { memset(P,Black,256);memset(Z,0x00,256/8);return; }

  L=Y+ScrollY;
  if(L>223) L-=223;
  T=ChrTab+((L&0xF8)<<3);
  Offset=(L&0x07)<<2;
  X2=(ScrollX&0xF8)>>2;
  Shift=ScrollX&0x07;
  P-=Shift;Z--;
  Z[0]=0x00;
  XR=NoRghScroll? 24:255;

  for(X1=0;X1<33;X1++)
  {
    /* If right 9 columns not scrolled, modify T and Offset */
    if(X1==XR) { T=ChrTab+((int)(Y&0xF8)<<3);Offset=(Y&0x07)<<2; }

    C=T+X2;                             /* C <== Address of tile #, attrs */
    J=C[1];                             /* J <== Tile attributes          */

    C = VRAM
      + (J&0x01? 0x2000:0x0000)         /* 0x01: Pattern table selection  */
      + ((int)C[0]<<5)                  /* Pattern offset in the table    */
      + (J&0x04? 28-Offset:Offset);     /* 0x04: Vertical flip            */

    R=J&0x08? SPal:XPal;                /* 0x08: Use sprite palette       */
    K=J&0x10;                           /* 0x10: Show in front of sprites */
    J&=0x02;                            /* 0x02: Horizontal flip          */

    I=M=J? Conv[C[0]]:C[0];
    L=((M&0x88)<<21)|((M&0x44)<<14)|((M&0x22)<<7)|(M&0x11);
    I|=M=J? Conv[C[1]]:C[1];
    L|=((M&0x88)<<22)|((M&0x44)<<15)|((M&0x22)<<8)|((M&0x11)<<1);
    I|=M=J? Conv[C[2]]:C[2];
    L|=((M&0x88)<<23)|((M&0x44)<<16)|((M&0x22)<<9)|((M&0x11)<<2);
    I|=M=J? Conv[C[3]]:C[3];
    L|=((M&0x88)<<24)|((M&0x44)<<17)|((M&0x22)<<10)|((M&0x11)<<3);

    P[0]=R[(L>>28)&0x0F];
    P[1]=R[(L>>20)&0x0F];
    P[2]=R[(L>>12)&0x0F];
    P[3]=R[(L>>4)&0x0F];
    P[4]=R[(L>>24)&0x0F];
    P[5]=R[(L>>16)&0x0F];
    P[6]=R[(L>>8)&0x0F];
    P[7]=R[L&0x0F];

    if(K)
    {
      L=(unsigned int)I<<Shift;
      Z[0]|=L>>8;Z[1]=L&0xFF;
    }
    else Z[1]=0x00;

    X2=(X2+2)&0x3F;
    P+=8;Z++;
  }

  /* Refresh sprites in this scanline */
  RefreshSprites(Y,ZBuf+1);
}

/** RefreshSprites() *****************************************/
/** Refresh sprites in a given scanline.                    **/
/*************************************************************/
void RefreshSprites(register byte Line,register byte *ZBuf)
{
  register byte H,J,*P,*C,*Z,Shift;
  register unsigned int L,M;
  int N,Y,X;

  /* If display turned off, we do not show sprites */
  if(!ScreenON) return;
  /* Find the number of the last displayed sprite */
  for(N=0;(N<64)&&(SprY(N)!=208);N++);
  /* Find the sprite height */
  H=Sprites16? 16:8;

  for(N--;N>=0;N--)
  {
    Y=SprY(N)+1;
    if(Y>240) Y-=256;
    if((Line>=Y)&&(Line<Y+H))
    {
      X=SprX(N);
      if(LeftSprites) X-=8;
      J=SprP(N);
      if(H>8) J&=0xFE;
      C=SprGen+((int)J<<5)+((int)(Line-Y)<<2);
      P=XBuf+WIDTH*(HEIGHT-192)/2+(WIDTH-256)/2+WIDTH*Line+X;
      Z=ZBuf+((X&0xFF)>>3);
      Shift=8-(X&0x07);

      J=M=C[0];
      L=((M&0x88)<<21)|((M&0x44)<<14)|((M&0x22)<<7)|(M&0x11);
      J|=M=C[1];
      L|=((M&0x88)<<22)|((M&0x44)<<15)|((M&0x22)<<8)|((M&0x11)<<1);
      J|=M=C[2];
      L|=((M&0x88)<<23)|((M&0x44)<<16)|((M&0x22)<<9)|((M&0x11)<<2);
      J|=M=C[3];
      L|=((M&0x88)<<24)|((M&0x44)<<17)|((M&0x22)<<10)|((M&0x11)<<3);
      J&=~((((int)Z[0]<<8)|Z[1])>>Shift);

      if(J)
      {
        if(J&0x80) P[0]=SPal[(L>>28)&0x0F];
        if(J&0x40) P[1]=SPal[(L>>20)&0x0F];
        if(J&0x20) P[2]=SPal[(L>>12)&0x0F];
        if(J&0x10) P[3]=SPal[(L>>4)&0x0F];
        if(J&0x08) P[4]=SPal[(L>>24)&0x0F];
        if(J&0x04) P[5]=SPal[(L>>16)&0x0F];
        if(J&0x02) P[6]=SPal[(L>>8)&0x0F];
        if(J&0x01) P[7]=SPal[L&0x0F];
      }
    }
  }
}

/** RefreshScreen() ******************************************/
/** Refresh screen. This function is called in the end if   **/
/** refresh cycle to show the screen.                       **/
/*************************************************************/
void RefreshScreen(void)
{
  register byte *P,J;

  /* Mask out left 8 pixels if needed  */
  if(LeftMask)
  {
    P=XBuf+WIDTH*(HEIGHT-192)/2+(WIDTH-256)/2;
    for(J=0;J<192;J++,P+=WIDTH)
      P[0]=P[1]=P[2]=P[3]=P[4]=P[5]=P[6]=P[7]=Black;
  }

  /* Copy image to screen */
  if(GameGear) PutImage((WIDTH-160)/2,(HEIGHT-144)/2,160,144);
  else         PutImage((WIDTH-256)/2,(HEIGHT-192)/2,256,192);
}
