/** MasterGear: portable MasterSystem/GameGear emulator ******/
/**                                                         **/
/**                           MG.c                          **/
/**                                                         **/
/** This file contains generic main() procedure statrting   **/
/** the emulation.                                          **/
/**                                                         **/
/** Copyright (C) Marat Fayzullin 1994,1995,1996            **/
/**     You are not allowed to distribute this software     **/
/**     commercially. Please, notify me, if you make any    **/
/**     changes to this file.                               **/
/*************************************************************/

#include "SMS.h"
#include "Help.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char *Options[]=
{ 
  "verbose","vperiod","uperiod","help","gg","sms",
  "autoa","noautoa","autob","noautob","jap",
  "engl","logsnd","delay","nodelay","trap",
  "sound","nosound","shm","noshm","saver","nosaver",
  "back","vsync","novsync",
  NULL
};

extern char *Title;      /* Program title                       */
extern int   UseSound;   /* Sound mode (#ifdef SOUND)           */
extern int   UseSHM;     /* Use SHM X extension (#ifdef MITSHM) */
extern int   SaveCPU;    /* Pause when inactive (#ifdef UNIX)   */
extern int   SyncScreen; /* Sync screen updates (#ifdef MSDOS)  */
extern char *BackName;   /* Background picture (#ifdef GIFLIB)  */


/** main() ***************************************************/
/** This is a main() function used in Unix and MSDOS ports. **/
/** It parses command line arguments, sets emulation        **/
/** parameters, and passes control to the emulation itself. **/
/*************************************************************/
int main(int argc,char *argv[])
{
  char *CartName,*P;
  int N,J,I,NoExt;

  TrapBadOps=0;
  VPeriod=8000;
  UPeriod=2;
  AutoA=AutoB=0;
  IntSync=1;
  Verbose=1;
  GameGear=0;
  Country=0;
  DelayedRD=0;
  CartName="CART.ROM";
  SndName=NULL;

  for(N=1,I=0,NoExt=0;N<argc;N++)
    if(*argv[N]!='-')
      switch(I++)
      {
        case 0: CartName=argv[N];break;
        default: printf("%s: Excessive filename '%s'\n",argv[0],argv[N]);
      }
    else
    {    
      for(J=0;Options[J];J++)
        if(!strcmp(argv[N]+1,Options[J])) break;
      switch(J)
      {
        case 0:  N++;
                 if(N<argc) Verbose=atoi(argv[N]);
                 else printf("%s: No verbose level supplied\n",argv[0]);
                 TrapBadOps=Verbose&0x04;
                 break;
  	case 1:  N++;
                 if(N>=argc)
                   printf("%s: No VBlank period supplied\n",argv[0]);
                 else
                 {
                   IntSync=1;VPeriod=atoi(argv[N]);
                   if((VPeriod<280)||(VPeriod>50000)) VPeriod=8000;
                 }
                 break;
  	case 2:  N++;
                 if(N>=argc)
                   printf("%s: No screen update period supplied\n",argv[0]);
                 else
                 {
                   UPeriod=atoi(argv[N]);
                   if((UPeriod<1)||(UPeriod>20)) UPeriod=2;
                 }
                 break;
	case 3:  printf
                 ("%s by Marat Fayzullin    (C)FMS 1994,1995,1996\n",Title);
                 for(J=0;HelpText[J];J++) puts(HelpText[J]);
                 return(0);
        case 4:  GameGear=1;NoExt=1;break;
        case 5:  GameGear=0;NoExt=1;break;
        case 6:  AutoA=1;break;
        case 7:  AutoA=0;break;
        case 8:  AutoB=1;break;
        case 9:  AutoB=0;break;
        case 10: Country=2;break;
        case 11: Country=1;break;
        case 12: N++;
                 if(N<argc) SndName=argv[N];
                 else printf("%s: No file for soundtrack logging\n",argv[0]);
                 break;
        case 13: DelayedRD=1;break;
        case 14: DelayedRD=0;break;
#ifdef DEBUG
        case 15: N++;
                 if(N>=argc)
                   printf("%s: No trap address supplied\n",argv[0]);
                 else
                   if(!strcmp(argv[N],"now")) Trace=1;
                   else sscanf(argv[N],"%hX",&Trap);
                 break;
#endif
#ifdef SOUND
        case 16: N++;
                 if(N>=argc) { UseSound=4;N--; }
                 else if(sscanf(argv[N],"%d",&UseSound)!=1)
                      { UseSound=4;N--; }
                 break;
        case 17: UseSound=0;break;
#endif
#ifdef UNIX
#ifdef MITSHM
        case 18: UseSHM=1;break;
        case 19: UseSHM=0;break;
#endif
        case 20: SaveCPU=1;break;
        case 21: SaveCPU=0;break;
#endif
#ifdef MSDOS
#ifdef GIFLIB
        case 22: N++;
                 if(N<argc) BackName=argv[N];
                 else printf("%s: No background picture supplied\n",argv[0]);
                 break;
#endif
        case 23: SyncScreen=1;break;
        case 24: SyncScreen=0;break;
#endif
        default: printf("%s: Wrong option '%s'\n",argv[0],argv[N]);
      }
    }

  if(!NoExt&&(P=strrchr(CartName,'.')))
  {
    if(!strcmp(P,".gg")||!strcmp(P,".GG"))   GameGear=1;
    if(!strcmp(P,".sms")||!strcmp(P,".SMS")) GameGear=0;
  }

  if(!InitMachine()) return(1);
  StartSMS(CartName);
  TrashSMS();
  TrashMachine();
  return(0);
}
